/*    
	This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.handler;

import java.text.CharacterIterator;
import java.text.StringCharacterIterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Entities {
	public static String escape(String input) {
		StringBuilder result = new StringBuilder();
		StringCharacterIterator iterator = new StringCharacterIterator(input);
		char c = iterator.current();
		while(c != CharacterIterator.DONE) {
			switch(c) {
				case '<':
					result.append("&lt;");
					break;
				case '>':
					result.append("&gt;");
					break;
				case '\"':
					result.append("&quot;");
					break;
				case '\'':
					result.append("&apos;");
					break;
				case '&':
					result.append("&amp;");
					break;
				default:
					result.append(c);
			}
			c = iterator.next();
		}
		return result.toString();
	}
	
	public static String unescape(String input) {
		Matcher matcher = pattern.matcher(input);
		StringBuffer output = new StringBuffer();
		while(matcher.find()) {
			String text = matcher.group().toLowerCase();
			if(text.compareTo("&lt;") == 0) {
				matcher.appendReplacement(output, Matcher.quoteReplacement("<"));
			} else
			if(text.compareTo("&gt;") == 0) {
				matcher.appendReplacement(output, Matcher.quoteReplacement(">"));
			} else
			if(text.compareTo("&quot;") == 0) {
				matcher.appendReplacement(output, Matcher.quoteReplacement("\""));
			} else
			if(text.compareTo("&apos;") == 0) {
				matcher.appendReplacement(output, Matcher.quoteReplacement("'"));
			} else
			if(text.compareTo("&amp;") == 0) {
				matcher.appendReplacement(output, Matcher.quoteReplacement("&"));
			}
		}
		matcher.appendTail(output);
		return output.toString();
	}	
	
	private static Pattern pattern = Pattern.compile("(&(lt|gt|quot|apos|amp);)", Pattern.CASE_INSENSITIVE);
}