/*    
	This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.handler;

import com.quigley.moose.mapping.naming.MappingNameGenerator;

@SuppressWarnings({"unchecked", "rawtypes"})
public class EnumHandler implements TypeHandler {
	public EnumHandler(Class enumClazz, MappingNameGenerator mappingNameGenerator) {
		this.enumClazz = enumClazz;
		this.mappingNameGenerator = mappingNameGenerator;
	}
	
	public Object getObject(String text) {
		return Enum.valueOf(enumClazz, text);
	}
	public String getText(Object object) {
		if(object != null) {
			return ((Enum) object).name();
		} else {
			return null;
		}
	}
	public String getXsdType(String namespacePrefix) {
		return getEnumXmlName(namespacePrefix);
	}
	
	public String getEnumXmlName(String namespacePrefix) {
		String className = mappingNameGenerator.classToXml(enumClazz);
		className += "-enum-type";
		if(namespacePrefix != null && namespacePrefix.trim().length() > 0) {
			className = namespacePrefix.trim() + ":" + className;
		}
		return className;
	}
	
	private Class enumClazz;
	private MappingNameGenerator mappingNameGenerator;
}