/*    
	This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.mapping;

import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;

import com.quigley.moose.handler.TypeHandler;

@SuppressWarnings({"unchecked", "rawtypes"})
public class InlineMappedListField extends MappedField {
	public InlineMappedListField() {
		super();
	}
	
	public void marshall(Object instance, Mapping mapping, MarshallingContext ctx) {
		List fieldList = (List) invokeGetter(instance);
		if(fieldList != null) {
			Class listElementClass = getParameterizedListType(instance.getClass());
			
			for(Object fieldObject : fieldList) {
				if(mapping.hasMappedType(listElementClass)) {
					MappedType mappedType = mapping.getMappedType(fieldObject.getClass());
					mappedType.marshall(fieldObject, xmlName, mapping, ctx);
				} else {
					if(mapping.hasTypeHandler(listElementClass)) {
						TypeHandler typeHandler = mapping.getTypeHandler(fieldObject.getClass());
						ctx.getOutput().append("<").append(mapping.namespaceify(xmlName)).append(">");
						ctx.getOutput().append(typeHandler.getText(fieldObject));
						ctx.getOutput().append("</").append(mapping.namespaceify(xmlName)).append(">");
					}
				}
			}
		}
	}

	public void unmarshall(Object instance, Element element, Mapping mapping) {
		if(!element.getNamespace().equals(mapping.getNamespace())) {
			throw new MappingException("Input namespace '" + element.getNamespace() + "' and mapped namespace differ.");
		}
		
		Class fieldElementClass = getParameterizedListType(instance.getClass());
		
		List list = (List) invokeGetter(instance);
		if(list == null) {
			list = new ArrayList();
		}
		if(mapping.hasMappedType(fieldElementClass)) {
			MappedType mappedType = mapping.getMappedType(fieldElementClass);
			list.add(mappedType.unmarshall(element, mapping));
		} else {
			if(mapping.hasTypeHandler(fieldElementClass)) {
				TypeHandler typeHandler = mapping.getTypeHandler(fieldElementClass);
				list.add(typeHandler.getObject(element.getText()));
			}
		}
		invokeSetter(instance, list);
	}
}