/*    
	This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.mapping;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.List;

import org.jdom.Element;

import com.quigley.moose.handler.TypeHandler;

@SuppressWarnings("rawtypes")
public class MappedArrayField extends MappedField {
	@Override
	public void marshall(Object instance, Mapping mapping, MarshallingContext ctx) {
		Object arrayObject = invokeGetter(instance);
		
		if(arrayObject != null) {
			ctx.getOutput().append("<").append(mapping.namespaceify(getXmlName())).append(">");
			
			for(int i = 0; i < Array.getLength(arrayObject); i++) {
				Object indexedObject = Array.get(arrayObject, i);
				if(mapping.hasMappedType(indexedObject.getClass())) {
					MappedType mappedType = mapping.getMappedType(indexedObject.getClass());
					mappedType.marshall(indexedObject, elementXmlName, mapping, ctx);
				} else {
					if(mapping.hasTypeHandler(indexedObject.getClass())) {
						TypeHandler typeHandler = mapping.getTypeHandler(indexedObject.getClass());
						ctx.getOutput().append("<").append(mapping.namespaceify(elementXmlName)).append(">");
						ctx.getOutput().append(typeHandler.getText(indexedObject));
						ctx.getOutput().append("</").append(mapping.namespaceify(elementXmlName)).append(">");
					}
				}				
			}
			
			ctx.getOutput().append("</").append(mapping.namespaceify(getXmlName())).append(">");
		}
	}

	@Override
	@SuppressWarnings("unchecked")
	public void unmarshall(Object instance, Element element, Mapping mapping) {
		if(!element.getNamespace().equals(mapping.getNamespace())) {
			throw new MappingException("Input namespace '" + element.getNamespace() + "' and mapped namespace differ.");
		}
		
		Class arrayElementClass = getFieldType(instance.getClass()).getComponentType();
		
		List list = new ArrayList();
		for(Element childElement : (List<Element>) element.getChildren(elementXmlName, mapping.getNamespace())) {
			if(mapping.hasMappedType(arrayElementClass)) {
				MappedType mappedType = mapping.getMappedType(arrayElementClass);
				list.add(mappedType.unmarshall(childElement, mapping));
			} else {
				if(mapping.hasTypeHandler(arrayElementClass)) {
					TypeHandler typeHandler = mapping.getTypeHandler(arrayElementClass);
					list.add(typeHandler.getObject(childElement.getText()));
				}
			}
		}
		
		Object newArray = Array.newInstance(arrayElementClass, list.size());
		int i = 0;
		for(Object arrayElement : list) {
			Array.set(newArray, i++, arrayElement);
		}
		
		invokeSetter(instance, newArray);		
	}
	
	public String getElementXmlName() {
		return elementXmlName;
	}
	public void setElementXmlName(String elementXmlName) {
		this.elementXmlName = elementXmlName;
	}

	private String elementXmlName;
}