/*    
	This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.mapping;

import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.jdom.Element;

import com.quigley.moose.handler.TypeHandler;

@SuppressWarnings("rawtypes")
public class MappedMapField extends MappedField {
	@Override
	public void marshall(Object instance, Mapping mapping, MarshallingContext ctx) {
		Object fieldObject = invokeGetter(instance);
		
		if(fieldObject != null) {
			ctx.getOutput().append("<").append(mapping.namespaceify(getXmlName())).append(">");

			Map fieldMap = (Map) fieldObject;
			for(Object keyObject : fieldMap.keySet()) {
				ctx.getOutput().append("<").append(mapping.namespaceify(pairXmlName)).append(">");
				
				if(mapping.hasMappedType(keyObject.getClass())) {
					MappedType mappedType = mapping.getMappedType(keyObject.getClass());
					mappedType.marshall(keyObject, keyXmlName, mapping, ctx);
				} else {
					if(mapping.hasTypeHandler(keyObject.getClass())) {
						TypeHandler typeHandler = mapping.getTypeHandler(keyObject.getClass());
						ctx.getOutput().append("<").append(mapping.namespaceify(keyXmlName)).append(">");
						ctx.getOutput().append(typeHandler.getText(keyObject));
						ctx.getOutput().append("</").append(mapping.namespaceify(keyXmlName)).append(">");
					}
				}
				
				Object valueObject = fieldMap.get(keyObject);
				if(mapping.hasMappedType(valueObject.getClass())) {
					MappedType mappedType = mapping.getMappedType(valueObject.getClass());
					mappedType.marshall(valueObject, valueXmlName, mapping, ctx);
				} else {
					if(mapping.hasTypeHandler(valueObject.getClass())) {
						TypeHandler typeHandler = mapping.getTypeHandler(valueObject.getClass());
						ctx.getOutput().append("<").append(mapping.namespaceify(valueXmlName)).append(">");
						ctx.getOutput().append(typeHandler.getText(valueObject));
						ctx.getOutput().append("</").append(mapping.namespaceify(valueXmlName)).append(">");
					}
				}
				
				ctx.getOutput().append("</").append(mapping.namespaceify(pairXmlName)).append(">");
			}
			
			ctx.getOutput().append("</").append(mapping.namespaceify(getXmlName())).append(">");
		}
	}

	@Override
	@SuppressWarnings("unchecked")
	public void unmarshall(Object instance, Element element, Mapping mapping) {
		if(!element.getNamespace().equals(mapping.getNamespace())) {
			throw new MappingException("Input namespace '" + element.getNamespace() + "' and mapped namespace differ.");
		}
		
		Type[] mapTypes = getParameterizedTypes(instance.getClass());
		if(mapTypes.length != 2) {
			throw new MappingException("Parameterized maps expect 2 parameterized types! I found " + mapTypes.length + "!");
		}
		Class keyClass = (Class<?>) mapTypes[0];
		Class valueClass = (Class<?>) mapTypes[1];
		
		Map map = new HashMap();
		for(Element pairElement : (List<Element>) element.getChildren(pairXmlName, mapping.getNamespace())) {
			Object keyObject = null;
			Element keyElement = pairElement.getChild(keyXmlName, mapping.getNamespace());
			if(mapping.hasMappedType(keyClass)) {
				MappedType mappedType = mapping.getMappedType(keyClass);
				keyObject = mappedType.unmarshall(keyElement, mapping);
			} else {
				if(mapping.hasTypeHandler(keyClass)) {
					TypeHandler typeHandler = mapping.getTypeHandler(keyClass);
					keyObject = typeHandler.getObject(keyElement.getText());
				}
			}

			Object valueObject = null;
			Element valueElement = pairElement.getChild(valueXmlName, mapping.getNamespace());
			if(mapping.hasMappedType(valueClass)) {
				MappedType mappedType = mapping.getMappedType(valueClass);
				valueObject = mappedType.unmarshall(valueElement, mapping);
			} else {
				if(mapping.hasTypeHandler(valueClass)) {
					TypeHandler typeHandler = mapping.getTypeHandler(valueClass);
					valueObject = typeHandler.getObject(valueElement.getText());
				}
			}
			
			map.put(keyObject, valueObject);
		}
		
		invokeSetter(instance, map);
	}
	
	public String getPairXmlName() {
		return pairXmlName;
	}
	public void setPairXmlName(String pairXmlName) {
		this.pairXmlName = pairXmlName;
	}

	public String getKeyXmlName() {
		return keyXmlName;
	}
	public void setKeyXmlName(String keyXmlName) {
		this.keyXmlName = keyXmlName;
	}

	public String getValueXmlName() {
		return valueXmlName;
	}
	public void setValueXmlName(String valueXmlName) {
		this.valueXmlName = valueXmlName;
	}

	protected String pairXmlName;
	protected String keyXmlName;
	protected String valueXmlName;
}