/*    
	This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.mapping;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.Namespace;

import com.quigley.moose.handler.TypeHandler;

public class MappedPropertyField extends MappedField {
	public void marshall(Object instance, Mapping mapping, MarshallingContext ctx) {
		Object fieldObject = invokeGetter(instance);
		if(fieldObject != null) {
			Class<?> fieldClass = getFieldType(instance.getClass());
			
			String outputName = xmlName;
			if(mapping.getXmlPrefix() != null) {
				outputName = mapping.getXmlPrefix() + ":" + outputName;
			}

			if(mapping.hasMappedType(fieldClass)) {
				mapping.marshall(fieldObject, getXmlName(), ctx);
			} else {
				ctx.getOutput().append("<").append(outputName).append(">");
				ctx.getOutput().append(mapping.getTypeHandler(fieldClass).getText(fieldObject));
				ctx.getOutput().append("</").append(outputName).append(">");
			}
		}
	}

	public void unmarshall(Object instance, Element element, Mapping mapping) {
		if(!element.getNamespace().equals(mapping.getNamespace())) {
			throw new MappingException("Input namespace '" + element.getNamespace() + "' and mapped namespace differ.");
		}

		Class<?> fieldClass = getFieldType(instance.getClass());
		
		// Select the correct field class, if xsi:type is present.
		Namespace xsiNamespace = Namespace.getNamespace("xsi", "http://www.w3.org/2001/XMLSchema-instance");
		Attribute xsiTypeAttribute = element.getAttribute("type", xsiNamespace);
		if(xsiTypeAttribute != null) {
			String actualTypeXmlName = xsiTypeAttribute.getValue().substring(0, xsiTypeAttribute.getValue().indexOf("-type"));
			MappedType actualMappedType = mapping.getMappedType(actualTypeXmlName);
			fieldClass = actualMappedType.getClazz();
		}
		
		Object fieldObject;
		if(mapping.hasMappedType(fieldClass)) {
			fieldObject = mapping.unmarshall(element, fieldClass);
		} else {
			TypeHandler typeHandler = mapping.getTypeHandler(fieldClass);
			fieldObject = typeHandler.getObject(element.getText());
		}

		invokeSetter(instance, fieldObject);
	}
}