/*    
	This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.mapping.provider.annotation.scannotation;

import java.io.IOException;
import java.net.URL;
import java.util.Set;

import org.scannotation.AnnotationDB;

import com.quigley.moose.mapping.provider.MappingProviderException;
import com.quigley.moose.mapping.provider.annotation.AnnotatedClassesProvider;

public class ScannotationClassesProvider implements AnnotatedClassesProvider {
	public ScannotationClassesProvider() {
		finderStrategy = new ClasspathUrlFinderStrategy();
		initialize();
	}
	public ScannotationClassesProvider(ScannotationFinderStrategy finderStrategy) {
		this.finderStrategy = finderStrategy;
		initialize();
	}
	
	protected void initialize() {
		try {
			annotationDB = new AnnotationDB();
			URL[] urls = finderStrategy.getClasspathURLs();
			annotationDB.scanArchives(urls);
			
		} catch(IOException ioe) {
			throw new MappingProviderException(ioe);
		}
	}
	
	public Set<String> getMappedClassNames() {
		return annotationDB.getAnnotationIndex().get(com.quigley.moose.mapping.provider.annotation.XML.class.getName());
	}
	
	protected AnnotationDB annotationDB;
	protected ScannotationFinderStrategy finderStrategy;
}