/*
    This file is part of Moose.

    Moose is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation, either version 3 of 
    the License, or (at your option) any later version.

    Moose is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public 
    License along with Moose.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.quigley.moose.schema;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.quigley.moose.mapping.Mapping;

public class SchemaGeneratorContext {
	public SchemaGeneratorContext(Mapping mapping) {
		this.mapping = mapping;
		out = new StringBuilder();
		currentIndentLevel = 0;
		
		this.mappedClasses = new ArrayList<Class<?>>(mapping.getMappedClasses());
		Collections.sort(mappedClasses, new ClassComparator());
		
		referencedEnums = new ArrayList<Class<?>>();
		listContainers = new ArrayList<ListContainer>();
		mapContainers = new ArrayList<MapContainer>();
	}

	public void indent() {
		currentIndentLevel++;
	}
	public void outdent() {
		currentIndentLevel--;
	}
	
	public StringBuilder append(String s) {
		for(int i = 0; i < currentIndentLevel; i++) {
			out.append("\t");
		}
		return out.append(s);
	}
	
	protected String getXmlPrefix() {
		String xmlPrefix = "";
		if(mapping.getXmlPrefix() != null) {
			xmlPrefix = mapping.getXmlPrefix() + ":";
		}
		return xmlPrefix;
	}
	
	public Mapping getMapping() {
		return mapping;
	}
	public void setMapping(Mapping mapping) {
		this.mapping = mapping;
	}

	public StringBuilder getOut() {
		return out;
	}
	public void setOut(StringBuilder out) {
		this.out = out;
	}

	public List<Class<?>> getMappedClasses() {
		return mappedClasses;
	}
	public void setMappedClasses(List<Class<?>> mappedClasses) {
		this.mappedClasses = mappedClasses;
	}
	
	public List<Class<?>> getReferencedEnums() {
		return referencedEnums;
	}
	public void setReferencedEnums(List<Class<?>> referencedEnums) {
		this.referencedEnums = referencedEnums;
	}
	
	public List<ListContainer> getListContainers() {
		return listContainers;
	}
	public void setListContainers(List<ListContainer> listContainers) {
		this.listContainers = listContainers;
	}
	public void addListContainer(ListContainer listContainer) {
		listContainers.add(listContainer);
	}

	public List<MapContainer> getMapContainers() {
		return mapContainers;
	}
	public void setMapContainers(List<MapContainer> mapContainers) {
		this.mapContainers = mapContainers;
	}
	public void addMapContainer(MapContainer mapContainer) {
		mapContainers.add(mapContainer);
	}

	private Mapping mapping;
	private StringBuilder out;
	private byte currentIndentLevel;
	private List<Class<?>> mappedClasses;
	private List<Class<?>> referencedEnums;
	private List<ListContainer> listContainers;
	private List<MapContainer> mapContainers;
}