package com.quigley.moose.spring;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.Writer;

import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLStreamReader;
import javax.xml.stream.XMLStreamWriter;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.stream.StreamSource;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.oxm.MarshallingFailureException;
import org.springframework.oxm.support.AbstractMarshaller;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.LexicalHandler;

import com.quigley.moose.mapping.Mapping;
import com.quigley.moose.mapping.MarshallingContext;
import com.quigley.moose.mapping.provider.MappingProvider;

public class MooseMarshaller extends AbstractMarshaller implements InitializingBean {
	@Override
	protected void marshalDomNode(Object o, Node node) {
		try {
			MarshallingContext ctx = new MarshallingContext();
			mapping.marshall(o, ctx);
			ByteArrayInputStream inputStream = new ByteArrayInputStream(ctx.getOutput().toString().getBytes());
			Transformer transformer = transformerFactory.newTransformer();
			transformer.transform(new StreamSource(inputStream), new DOMResult(node));

		} catch(Exception e) {
			throw new MarshallingFailureException("Unable to marshall.", e);
		}
	}

	@Override
	protected void marshalOutputStream(Object o, OutputStream outputStream) throws IOException {
		MarshallingContext ctx = new MarshallingContext();
		mapping.marshall(o, ctx);
		outputStream.write(ctx.getOutput().toString().getBytes());
	}

	@Override
	protected void marshalSaxHandlers(Object arg0, ContentHandler arg1, LexicalHandler arg2) {
		throw new UnsupportedOperationException("marshalSaxHandlers()");
	}

	@Override
	protected void marshalWriter(Object o, Writer writer) throws IOException {
		MarshallingContext ctx = new MarshallingContext();
		mapping.marshall(o, ctx);
		writer.write(ctx.getOutput().toString());
	}

	@Override
	protected void marshalXmlEventWriter(Object arg0, XMLEventWriter arg1) {
		throw new UnsupportedOperationException("marshalXmlEventWriter()");
	}

	@Override
	protected void marshalXmlStreamWriter(Object arg0, XMLStreamWriter arg1) {
		throw new UnsupportedOperationException("marshalXmlStreamWriter()");
	}

	@Override
	protected Object unmarshalDomNode(Node node) {
		if(node.getNodeType() == Node.ELEMENT_NODE) {
			return mapping.unmarshall((Element) node);
		} else {
			throw new MarshallingFailureException("I only know how to marshall starting from an element node!");
		}
	}

	@Override
	protected Object unmarshalInputStream(InputStream inputStream) throws IOException {
		return mapping.unmarshall(inputStream);
	}

	@Override
	protected Object unmarshalReader(Reader reader) throws IOException {
		return mapping.unmarshall(reader);
	}

	@Override
	protected Object unmarshalSaxReader(XMLReader arg0, InputSource arg1) throws IOException {
		throw new UnsupportedOperationException("unmarshalSaxReader()");
	}

	@Override
	protected Object unmarshalXmlEventReader(XMLEventReader arg0) {
		throw new UnsupportedOperationException("unmarshalXmlEventReader()");
	}

	@Override
	protected Object unmarshalXmlStreamReader(XMLStreamReader arg0) {
		throw new UnsupportedOperationException("unmarshalXmlStreamReader()");
	}

	@SuppressWarnings("rawtypes")
	public boolean supports(Class clazz) {
		if(mapping.hasMappedType(clazz)) {
			return true;
		}
		if(mapping.hasTypeHandler(clazz)) {
			return true;
		}
		return false;
	}
	
	public void afterPropertiesSet() throws Exception {
		if(mappingProvider != null) {
			mapping = mappingProvider.getMapping();
		}
	}

	public Mapping getMapping() {
		return mapping;
	}
	public void setMapping(Mapping mapping) {
		this.mapping = mapping;
	}

	public MappingProvider getMappingProvider() {
		return mappingProvider;
	}
	public void setMappingProvider(MappingProvider mappingProvider) {
		this.mappingProvider = mappingProvider;
	}

	protected Mapping mapping;
	protected MappingProvider mappingProvider;
	
	protected static TransformerFactory transformerFactory = TransformerFactory.newInstance();
}